/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_FLANGE_H_
#define __GDAM_FLANGE_H_

#include "../gdambufferingfilter.h"
#include "../gdamecho.h"

typedef struct _GdamFlange GdamFlange;
typedef struct _GdamFlangeClass GdamFlangeClass;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

GtkType gdam_flange_get_type();
#define GDAM_TYPE_FLANGE			(gdam_flange_get_type ())
#define GDAM_FLANGE(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_FLANGE, GdamFlange))
#define GDAM_FLANGE_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_FLANGE, GdamFlangeClass))
#define GDAM_IS_FLANGE(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_FLANGE))
#define GDAM_IS_FLANGE_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_FLANGE))

struct _GdamFlangeClass {
	GdamBufferingFilterClass	buffering_filter_class;
};
struct _GdamFlange {
	GdamBufferingFilter		buffering_filter;

	/* At each step, multiply the echo by a number
	 * between MIN_ECHO_FACTOR and MIN_ECHO_FACTOR+DELTA_ECHO_FACTOR,
	 * which are linearly mapped from samples
	 * from modulator. */
	GdamSource*			modulator;

	GdamFixed			min_echo_factor;
	GdamFixed			delta_echo_factor;

	int				num_echoes;
	GdamEchoElement*		echoes;
	int				max_echo_delay;
};

void gdam_flange_set_modulator(GdamFlange* flange, GdamSource* mod);
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
